package com.huiju.miniapp.common;

import java.io.Serializable;

/**
 * 接口统一状态返回BEAN.
 */
public class ResponseBean<T> implements Serializable {

    private static final long serialVersionUID = 3593827217136880822L;

    public static final int CODE_SUCCESS = 0;

    private int code;
    private String message;
    private T data;

    public static <T> ResponseBean success(T data, String ...msgs) {
        ResponseBean responseBean = new ResponseBean();
        responseBean.code = ResponseBean.CODE_SUCCESS;
        responseBean.data = data;

        if (null != msgs && msgs.length > 0) {
            responseBean.message = msgs[0];
        }

        return responseBean;
    }

    public static <T> ResponseBean error(String msg, int code, T ...datas) {
        ResponseBean responseBean = new ResponseBean();
        responseBean.code = code;
        responseBean.message = msg;

        if (null != datas && datas.length > 0) {
            responseBean.data = datas[0];
        }

        return responseBean;
    }

    public T getData() {
        return this.data;
    }
    public int getCode() {
        return this.code;
    }
    public String getMessage() {
        return this.message;
    }

    @Override
    public String toString() {
        return  "{ code: "+code+", message: "+message+", data: <T> }";
    }
}
