package com.huiju.miniapp.controller;

/**
 * 集中处理 管理端系统 接口
 */

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.miniapp.common.Encrypt;
import com.huiju.miniapp.common.InitWxMaService;
import com.huiju.miniapp.common.ResponseBean;
import com.huiju.miniapp.entity.MiniappConfiger;
import com.huiju.miniapp.entity.SystemConfiger;
import com.huiju.miniapp.service.IMiniappConfigerService;
import com.huiju.miniapp.service.ISystemConfigerService;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.time.LocalDateTime;

@RestController
@RequestMapping("/api")
public class CommController {
    @Autowired
    ISystemConfigerService iSystemConfigerService;

    @Autowired
    IMiniappConfigerService iMiniappConfigerService;

    @PostMapping("/admin/login")
    public ResponseBean login(@RequestBody String paramStr, HttpServletRequest request) {
        if (null == paramStr || "".equals(paramStr.trim())) {
            return ResponseBean.error("账户或密码不能为空", HttpStatus.SC_BAD_REQUEST);
        }

        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("登录参数输出格式非法", HttpStatus.SC_BAD_REQUEST);
        }

        HttpSession httpSession = request.getSession();
        String user = params.getString("user");
        String password = params.getString("password");
        if (null == user || null == password) {
            return ResponseBean.error("账户或密码不能为空", HttpStatus.SC_BAD_REQUEST);
        }

        SystemConfiger systemConfiger = iSystemConfigerService.getByUser(user);
        if (null == systemConfiger) {
            return ResponseBean.error("账户或密码不正确", HttpStatus.SC_BAD_REQUEST);
        }

        if (!checkPass(user, password, systemConfiger.getPassword())) {
            return ResponseBean.error("账户或密码不正确", HttpStatus.SC_BAD_REQUEST);
        }

        // 登录成功
        httpSession.setAttribute("token", user);
        systemConfiger.setPassword("");
        return ResponseBean.success(systemConfiger);
    }

    @PostMapping("/admin/logout")
    public ResponseBean logout(HttpServletRequest request) {
        HttpSession httpSession = request.getSession();
        httpSession.removeAttribute("token");
        return ResponseBean.success("");
    }

    @GetMapping("/admin/miniapp")
    public ResponseBean list(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer pageSize
    ) {
        IPage<MiniappConfiger> pg = new Page<>(page, pageSize);
        QueryWrapper<MiniappConfiger> wrapper = new QueryWrapper<>();
        wrapper.orderByDesc("create_date");

        IPage<MiniappConfiger> result = iMiniappConfigerService.page(pg, wrapper);
        return ResponseBean.success(result);
    }


    @PostMapping("/admin/miniapp")
    public ResponseBean save(@RequestBody MiniappConfiger miniappConfiger) {
        return iMiniappConfigerService.registe(miniappConfiger);
    }

    @PutMapping("/admin/miniapp/{id}")
    public ResponseBean update(@RequestBody MiniappConfiger miniappConfiger) {
        boolean success = iMiniappConfigerService.updateById(miniappConfiger);

        if (success) {
            InitWxMaService.resetService(miniappConfiger);
            return ResponseBean.success(miniappConfiger);
        } else {
            return ResponseBean.error("更新失败, 请重试", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }
    }

    private boolean checkPass(String user, String pass, String compare) {
        String target = Encrypt.md5(pass, user);
        return target.equals(compare);
    }
}
