package com.huiju.miniapp.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaTemplateMessage;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huiju.miniapp.common.InitWxMaService;
import com.huiju.miniapp.common.ResponseBean;
import com.huiju.miniapp.common.TemplateMsgUtil;
import org.apache.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/api")
public class MessageController {

    @PostMapping("/{appid}/tpl/{tplId}/form/{formId}/to/{user}")
    public ResponseBean send(
            @PathVariable String appid,
            @PathVariable String tplId,
            @PathVariable String formId,
            @PathVariable String user,
            @RequestBody String paramsStr
    ) {
        if (null == paramsStr || "".equals(paramsStr.trim())) {
            return ResponseBean.error("发送内容不能为空", HttpStatus.SC_NOT_ACCEPTABLE);
        }

        JSONObject params = JSONObject.parseObject(paramsStr);
        String page = params.getString("page");
        JSONArray data = params.getJSONArray("data");

        if (null == page) page = "";
        if (null == data || data.isEmpty()) {
            return ResponseBean.error("发送模板数据不能为空", HttpStatus.SC_NOT_ACCEPTABLE);
        }

        List<Object> dt = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            dt.add(data.get(i));
        }

        WxMaService wxMaService = InitWxMaService.getMaService(appid);
        if (null == wxMaService) return ResponseBean.error("当前小程序未注册接口", HttpStatus.SC_BAD_REQUEST);

        try {
            WxMaTemplateMessage message = TemplateMsgUtil.create(tplId, formId, user, page, dt);
            wxMaService.getMsgService().sendTemplateMsg(message);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error("发送模板消息失败" + e.getMessage(), HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        return ResponseBean.success("模板消息发送成功");
    }
}
