package com.huiju.miniapp.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaMessage;
import cn.binarywang.wx.miniapp.constant.WxMaConstants;
import com.huiju.miniapp.common.HttpUtil;
import com.huiju.miniapp.common.InitWxMaService;
import com.huiju.miniapp.common.ResponseBean;
import com.huiju.miniapp.entity.MiniappConfiger;
import com.huiju.miniapp.service.IMiniappConfigerService;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Objects;

@RestController
@RequestMapping("/api")
public class MiniAppController {

    @Autowired
    IMiniappConfigerService miniappConfigerService;

    @PostMapping("/{appid}/signup")
    public ResponseBean registe(@PathVariable String appid, @RequestBody MiniappConfiger miniappConfiger) {
        if (!appid.equals(miniappConfiger.getAppid())) {
            return ResponseBean.error("注册参数与请求路径不匹配", HttpStatus.SC_BAD_REQUEST);
        }

        // 印象中 secret 必须长度 43
        if (null == miniappConfiger.getSecret() || miniappConfiger.getSecret().length() < 5) {
            return ResponseBean.error("小程序 secret 参数为空或非法", HttpStatus.SC_BAD_REQUEST);
        }

        ResponseBean resp = miniappConfigerService.registe(miniappConfiger);
        if (resp.getCode() == ResponseBean.CODE_SUCCESS) {
            return ResponseBean.success("服务注册成功");
        }

        return resp;
    }

    /**
     * 小程序接入
     * 没有进行 signature 校验, 直接返回 echostr
     * @param echostr
     * @return
     */
    @GetMapping("/{appid}/portal")
    public String access(String echostr) {
        return echostr;
    }

    @PostMapping("/{appid}/portal")
    public String receiveMsg(@PathVariable String appid,
                             @RequestBody String requestBody,
                             @RequestParam(name = "msg_signature", required = false) String msgSignature,
                             @RequestParam(name = "encrypt_type", required = false) String encryptType,
                             @RequestParam(name = "signature", required = false) String signature,
                             @RequestParam("timestamp") String timestamp,
                             @RequestParam("nonce") String nonce) {
        final WxMaService wxService = InitWxMaService.getMaService(appid);
        if (null == wxService) {
            throw new RuntimeException("未注册的小程序服务：" + encryptType);
        }

        MiniappConfiger miniappConfiger = miniappConfigerService.getById(appid);
        if (null == miniappConfiger) {
            return "success";
        }

        // 如果小程序服务端没有设置回调接口, 就放弃接收到的消息
        String cbURL = miniappConfiger.getCallbackUrl();
        if (null == cbURL || "".equals(cbURL.trim())) {
            return "success";
        } else {
            cbURL += "/message";
        }

        final boolean isJson = Objects.equals(wxService.getWxMaConfig().getMsgDataFormat(),
                WxMaConstants.MsgDataFormat.JSON);
        if (null == encryptType || "".equals(encryptType.trim())) {
            // 明文传输的消息
            WxMaMessage inMessage;
            if (isJson) {
                inMessage = WxMaMessage.fromJson(requestBody);
            } else {//xml
                inMessage = WxMaMessage.fromXml(requestBody);
            }

            HttpUtil.asyncPost(cbURL, inMessage.toJson());
            return "success";
        }

        if ("aes".equals(encryptType)) {
            // 是aes加密的消息
            WxMaMessage inMessage;
            if (isJson) {
                inMessage = WxMaMessage.fromEncryptedJson(requestBody, wxService.getWxMaConfig());
            } else {//xml
                inMessage = WxMaMessage.fromEncryptedXml(requestBody, wxService.getWxMaConfig(),
                        timestamp, nonce, msgSignature);
            }

            HttpUtil.asyncPost(cbURL, inMessage.toJson());
            return "success";
        }

        throw new RuntimeException("不可识别的加密类型：" + encryptType);
    }
}
