package com.huiju.miniapp.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import com.huiju.miniapp.common.InitWxMaService;
import com.huiju.miniapp.common.ResponseBean;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/miniapp")
public class UserController {

    @GetMapping("/{appid}/login")
    public ResponseBean login(@PathVariable String appid, String code) {
        if (null == code || "".equals(code.trim())) {
            return ResponseBean.error("code 不能为空", HttpStatus.SC_BAD_REQUEST);
        }

        final WxMaService wxService = InitWxMaService.getMaService(appid);

        try {
            WxMaJscode2SessionResult session = wxService.getUserService().getSessionInfo(code);
            return ResponseBean.success(session);
        } catch (WxErrorException e) {
            e.printStackTrace();
            return ResponseBean.error("依据 code 解析 session 失败, " + e.getMessage(), HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping("/{appid}/user/info")
    public ResponseBean info(@PathVariable String appid, String sessionKey,
                       String signature, String rawData, String encryptedData, String iv) {

        final WxMaService wxService = InitWxMaService.getMaService(appid);

        // 用户信息校验
        if (!wxService.getUserService().checkUserInfo(sessionKey, rawData, signature)) {
            return  ResponseBean.error("校验 session 加密数据失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        // 解密用户信息
        WxMaUserInfo userInfo = wxService.getUserService().getUserInfo(sessionKey, encryptedData, iv);
        return ResponseBean.success(userInfo);
    }

    @GetMapping("/{appid}/user/phone")
    public ResponseBean phone(@PathVariable String appid, String sessionKey, String signature,
                        String rawData, String encryptedData, String iv) {
        final WxMaService wxService = InitWxMaService.getMaService(appid);

        // 用户信息校验
        if (!wxService.getUserService().checkUserInfo(sessionKey, rawData, signature)) {
            return ResponseBean.error("校验 session 加密数据失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        // 解密
        WxMaPhoneNumberInfo phoneNoInfo = wxService.getUserService().getPhoneNoInfo(sessionKey, encryptedData, iv);
        return ResponseBean.success(phoneNoInfo);
    }
}
