package com.huiju.miniapp.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.huiju.miniapp.common.Encrypt;
import com.huiju.miniapp.common.ResponseBean;
import com.huiju.miniapp.entity.MiniappConfiger;
import com.huiju.miniapp.service.IMiniappConfigerService;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

@Component
public class AccessInterceptor implements HandlerInterceptor {
    @Autowired
    IMiniappConfigerService iMiniappConfigerService;

    // 正则白名单
    private String[] whiteList = {
            "/api/admin/login",
            "/api/[^/]*/signup",
    };

    /*
     * 视图渲染之后的操作
     */
    @Override
    public void afterCompletion(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, Exception arg3)
            throws Exception {

    }

    /*
     * 处理请求完成后视图渲染之前的处理操作
     */
    @Override
    public void postHandle(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, ModelAndView arg3)
            throws Exception {

    }

    /*
     * 进入controller层之前拦截请求
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        // 白名单跳过
        if (inWhiteList(request)) {
            return true;
        }

        return isPcAdmin(request) ? checkSession(request, response, obj) : checkMiniApp(request, response, obj);
    }

    private boolean checkMiniApp(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {

        // 消息头必须包含 authorization 字段
        String token = request.getHeader("x-token");
        if (null == token || "".equals(token)) {
            ResponseBean resp = ResponseBean.error("请先进行系统登录操作", HttpStatus.SC_UNAUTHORIZED);

            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
            return false;
        }

        String uri = request.getRequestURI();
        String[] pathItems = uri.split("/");
        if (null == pathItems || pathItems.length < 2) {
            ResponseBean resp = ResponseBean.error("调用接口非法", HttpStatus.SC_BAD_REQUEST);

            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
            return false;
        }

        String appid = pathItems[2];
        MiniappConfiger configer = iMiniappConfigerService.getById(appid);
        if (null == configer) {
            ResponseBean resp = ResponseBean.error("请先进行服务注册", HttpStatus.SC_UNAUTHORIZED);

            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
            return false;
        }

        if (!Encrypt.md5(configer.getSecret(), appid).equals(token)) {
            ResponseBean resp = ResponseBean.error("权限校验失败, 非法调用", HttpStatus.SC_UNAUTHORIZED);

            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(resp));
            return false;
        }

        return true;
    }

    private boolean checkSession(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {

        HttpSession httpSession = request.getSession();
        String token = (String) httpSession.getAttribute("token");

        // 如果未登录
        if (null == token || "".equals(token)) {
            Map error = new HashMap<String, Object>();
            error.put("code", HttpStatus.SC_UNAUTHORIZED);
            error.put("message", "请先登录系统");
            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(error));
            return false;
        }
        return true;
    }

    private boolean isPcAdmin(HttpServletRequest request) {
        String uri = request.getRequestURI();
        return uri.startsWith("/api/admin");
    }

    private boolean inWhiteList(HttpServletRequest request) {
        String requestURI = request.getRequestURI();

        for (String it : whiteList) {
            if (Pattern.matches(it, requestURI)) {
                return true;
            }
        }

        return false;
    }

}

